package com.yydcdut.note.mvp.v.setting.impl;

import android.content.Intent;
import android.net.Uri;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import com.yydcdut.note.NoteApplication;
import com.yydcdut.note.R;
import com.yydcdut.note.injector.component.DaggerActivityComponent;
import com.yydcdut.note.injector.module.ActivityModule;
import com.yydcdut.note.mvp.p.setting.IFeedbackPresenter;
import com.yydcdut.note.mvp.p.setting.impl.AboutAppPresenterImpl;
import com.yydcdut.note.mvp.v.BaseActivity;
import com.yydcdut.note.mvp.v.setting.IAboutAppView;
import com.yydcdut.note.utils.LollipopCompat;

import javax.inject.Inject;

import butterknife.ButterKnife;
import butterknife.OnClick;

/**
 * Created by yuyidong on 15/9/2.
 */
public class AboutAppActivity extends BaseActivity implements IAboutAppView {
    @Inject
    AboutAppPresenterImpl mAboutAppPresenter;

    @Override
    public boolean setStatusBar() {
        return true;
    }

    @Override
    public int setContentView() {
        return R.layout.activity_about;
    }

    @Override
    public void initInjector() {
        mActivityComponent = DaggerActivityComponent.builder()
                .activityModule(new ActivityModule(this))
                .applicationComponent(((NoteApplication) getApplication()).getApplicationComponent())
                .build();
        mActivityComponent.inject(this);
    }

    @Override
    public void initUiAndListener() {
        ButterKnife.bind(this);
        mAboutAppPresenter.attachView(this);
        initToolBarUI();
    }

    private void initToolBarUI() {
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        toolbar.setTitle(getResources().getString(R.string.about_setting));
        setSupportActionBar(toolbar);
        toolbar.setNavigationIcon(R.drawable.ic_arrow_back_white_24dp);
        LollipopCompat.setElevation(toolbar, getResources().getDimension(R.dimen.ui_elevation));
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                break;
        }
        return true;
    }

    @OnClick(R.id.layout_update)
    public void clickUpdate(View v) {
        mAboutAppPresenter.doUpdateVersion();
    }

    @OnClick(R.id.layout_contact)
    public void clickContact(View v) {
        mAboutAppPresenter.doFeedback();
    }

    @OnClick(R.id.layout_share)
    public void clickShare(View v) {
        mAboutAppPresenter.doShare();
    }

    @OnClick(R.id.layout_github)
    public void clickGitHub(View v) {
        mAboutAppPresenter.gotoGithub();
    }

    @Override
    public void showVersion(String version) {
        ((TextView) findViewById(R.id.txt_version)).setText(getResources().getString(R.string.version)
                + " " + version);
    }

    @Override
    public void updateApk() {
        Uri uri = Uri.parse("http://a.app.qq.com/o/simple.jsp?pkgname=com.yydcdut.note");
        Intent intent = new Intent(Intent.ACTION_VIEW, uri);
        startActivity(intent);
    }

    @Override
    public void jump2FeedbackActivity() {
        Intent data = new Intent(this, FeedbackActivity.class);
        data.putExtra(IFeedbackPresenter.TYPE, IFeedbackPresenter.TYPE_CONTACT);
        startActivity(data);
    }

    @Override
    public void share() {
        Intent sendIntent = new Intent();
        sendIntent.setAction(Intent.ACTION_SEND);
        sendIntent.putExtra(Intent.EXTRA_TEXT, getResources().getText(R.string.about_share_content));
        sendIntent.setType("text/plain");
        startActivity(Intent.createChooser(sendIntent, getResources().getString(R.string.share)));
    }

    @Override
    public void viewGitHub() {
        Uri url = Uri.parse("https://github.com/yydcdut/PhotoNoter");
        Intent intent = new Intent(Intent.ACTION_VIEW, url);
        startActivity(intent);
    }
}
